/* *******************************************************************

   Minimum Example for a webserver on the ESP8266 / ESP32

   Hardware
   This sketch supports ESP8266 and ESP32 based boards

   Software
   The Software was last tested with
   - IDE 1.8.19
   - ESP32 Core 2.0.6, 1.0.6    (2.0.8 can have issues)
   - ESP8266 Core 3.0.2, 2.7.4

   Short
   - simple webserver with some pages
   - Stylesheet (css) optimized for mobile devices
   - ArduinoOTA upload with Arduino IDE

   Next
   - see an advanced webserver tutorial on: https://werner.rothschopf.net/

   Open Tasks
   - to be done

   by noiasca

   Version
   2023-08-04 HTML cleanup
   2023-07-27 First strip down Version based on the generic webserver

***************************************************************** */

#ifdef ARDUINO_ARCH_ESP8266            // libraries if you compile for a ESP8266
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>          // for the webserver
#include <ESP8266mDNS.h>               // Bonjour/multicast DNS, finds the device on network by name
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#endif
#ifdef ARDUINO_ARCH_ESP32              // libraries if you compile for a ESP32
#include <WiFi.h>
#include <WiFiClient.h>
#include <WebServer.h>                 // for the webserver
#include <ESPmDNS.h>                   // Bonjour/multicast DNS, finds the device on network by name   
#include <ArduinoOTA.h>                // OTA Upload via ArduinoIDE
#endif

#define VERSION "1.0.0"                // the version of this sketch

/* *******************************************************************
         define which board variant should be compiled
         set the number to one of your defined boards
 ********************************************************************/

#define USE_BOARD 1                   // the actual board to compile:
//                                         1 a generic demo / base configuration for your boards
#include "config1.h"                  // this file holds several options for that specific board

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>               // if you have an external file with your credentials you can use it - remove before upload

#ifndef STASSID                        // either use an external .h file containing STASSID and STAPSK - or 
#define STASSID "your-ssid"            // ... modify these line to your SSID
#define STAPSK  "your-password"        // ... and set your WIFI password
#endif

const char* ssid = STASSID;
const char* password = STAPSK;

/* *******************************************************************
         Globals - Variables and constants
 ********************************************************************/

#ifdef ARDUINO_ARCH_ESP8266
ESP8266WebServer server(80);           // an instance for the webserver
#else
WebServer server(80);
#endif

// some user variables
int anInteger = 123;
float aFloat = 12.34;
String aString = "abcd";
/* *******************************************************************
         S E T U P
 ********************************************************************/

void setup(void) {
  Serial.begin(115200);
  Serial.print(F("\n" TXT_BOARDNAME "\nVersion: " VERSION " Board "));
  Serial.println(USE_BOARD);
  Serial.print  (__DATE__);
  Serial.print  (" ");
  Serial.println(__TIME__);
  //printDefinitions();// debug only

  if (wifiActivityPin < 255)
    pinMode(wifiActivityPin, OUTPUT);
  char myhostname[16] {"esp"};         // The sketch will add the board ID. Take care about the total length of the char array (including null)
  char val[8];
  itoa(USE_BOARD, val, 10);            // before we can concatenate the precompiler integer, we have to convert it to char array
  strcat(myhostname, val);
  WiFi.hostname(myhostname);
#ifdef ARDUINO_ARCH_ESP32
  WiFi.setSleep(false);                // disable powersafe https://github.com/espressif/arduino-esp32/issues/1484
#endif
  Serial.print("try ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  // Wait for connection
  int8_t i = 0;  // just a counter to be able generate a line feed
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print('.');
    if (wifiActivityPin < 255) 
      digitalRead(wifiActivityPin) == LOW ? digitalWrite(wifiActivityPin, HIGH) : digitalWrite(wifiActivityPin, LOW); // blink one pin as indicator
    i++;
    if (i >= 40) {
      Serial.println();
      i = 0;
    }
  }
  Serial.print(F("\nConnected to "));
  Serial.println(ssid);
  Serial.print(F("IP address: "));
  Serial.println(WiFi.localIP());
  if (MDNS.begin(myhostname)) {
    Serial.println(F("MDNS responder started"));
  }
  if (wifiActivityPin < 255) digitalWrite(wifiActivityPin, wifiActivityPin);
  //define the pages and other content for the webserver
  server.on("/",      handlePage);               // send root page
  server.on("/0.htm", handlePage);               // the page can use the same handle
  server.on("/1.htm", handlePage1);              // another page, another handle
  server.on("/f.css", handleCss);                // a stylesheet
  server.on("/favicon.ico", handle204);          // not available, send 204
  server.onNotFound(handleNotFound);             // show a typical HTTP Error 404 page
  server.begin();                                // start the webserver
  Serial.println(F("HTTP server started\n"));

  ArduinoOTASetup();
}

/* *******************************************************************
         M A I N L O O P
 ********************************************************************/

void loop(void) {
  server.handleClient();               // call the webserver
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
  // do your calculation, sensor readings and update your global variables
  anInteger = random(0, 1000);
  aFloat = random(0, 1000) / 100.0;

  // add displays, serial outputs here
}
